@@ -1,4 +1,5 @@
 // #![warn(missing_docs)]
+use std::sync::Arc;
 
 use crate::{
     animation_manager::AnimationManager, data::output::PlatformOutput, frame_state::FrameState,
@@ -17,7 +18,7 @@ impl Default for WrappedTextureManager {
         // Will be filled in later
         let font_id = tex_mngr.alloc(
             "egui_font_texture".into(),
-            epaint::AlphaImage::new([0, 0]).into(),
+            epaint::FontImage::new([0, 0]).into(),
         );
         assert_eq!(font_id, TextureId::default());
 
@@ -48,6 +49,7 @@ struct ContextImpl {
 
     /// While positive, keep requesting repaints. Decrement at the end of each frame.
     repaint_requests: u32,
+    request_repaint_callbacks: Option<Box<dyn Fn() + Send + Sync>>,
 }
 
 impl ContextImpl {
@@ -113,7 +115,7 @@ impl ContextImpl {
 /// [`Context`] is cheap to clone, and any clones refers to the same mutable data
 /// ([`Context`] uses refcounting internally).
 ///
-/// All methods are marked `&self`; `Context` has interior mutability (protected by a mutex).
+/// All methods are marked `&self`; [`Context`] has interior mutability (protected by a mutex).
 ///
 ///
 /// You can store
@@ -122,7 +124,7 @@ impl ContextImpl {
 ///
 /// ``` no_run
 /// # fn handle_platform_output(_: egui::PlatformOutput) {}
-/// # fn paint(textures_detla: egui::TexturesDelta, _: Vec<egui::ClippedMesh>) {}
+/// # fn paint(textures_detla: egui::TexturesDelta, _: Vec<egui::ClippedPrimitive>) {}
 /// let mut ctx = egui::Context::default();
 ///
 /// // Game loop:
@@ -137,8 +139,8 @@ impl ContextImpl {
 ///         });
 ///     });
 ///     handle_platform_output(full_output.platform_output);
-///     let clipped_meshes = ctx.tessellate(full_output.shapes); // create triangles to paint
-///     paint(full_output.textures_delta, clipped_meshes);
+///     let clipped_primitives = ctx.tessellate(full_output.shapes); // create triangles to paint
+///     paint(full_output.textures_delta, clipped_primitives);
 /// }
 /// ```
 #[derive(Clone)]
@@ -222,9 +224,16 @@ impl Context {
 
     // ---------------------------------------------------------------------
 
-    /// If the given [`Id`] is not unique, an error will be printed at the given position.
-    /// Call this for [`Id`]:s that need interaction or persistence.
-    pub(crate) fn register_interaction_id(&self, id: Id, new_rect: Rect) {
+    /// If the given [`Id`] has been used previously the same frame at at different position,
+    /// then an error will be printed on screen.
+    ///
+    /// This function is already called for all widgets that do any interaction,
+    /// but you can call this from widgets that store state but that does not interact.
+    ///
+    /// The given [`Rect`] should be approximately where the widget will be.
+    /// The most important thing is that [`Rect::min`] is approximately correct,
+    /// because that's where the warning will be painted. If you don't know what size to pick, just pick [`Vec2::ZERO`].
+    pub fn check_for_id_clash(&self, id: Id, new_rect: Rect, what: &str) {
         let prev_rect = self.frame_state().used_ids.insert(id, new_rect);
         if let Some(prev_rect) = prev_rect {
             // it is ok to reuse the same ID for e.g. a frame around a widget,
@@ -243,7 +252,8 @@ impl Context {
                         painter.error(
                             rect.left_bottom() + vec2(2.0, 4.0),
                             "ID clashes happens when things like Windows or CollapsingHeaders share names,\n\
-                             or when things like ScrollAreas and Resize areas aren't given unique id_source:s.",
+                             or when things like Plot and Grid:s aren't given unique id_source:s.\n\n\
+                             Sometimes the solution is to use ui.push_id.",
                         );
                     }
                 }
@@ -252,10 +262,19 @@ impl Context {
             let id_str = id.short_debug_format();
 
             if prev_rect.min.distance(new_rect.min) < 4.0 {
-                show_error(new_rect.min, format!("Double use of ID {}", id_str));
+                show_error(
+                    new_rect.min,
+                    format!("Double use of {} ID {}", what, id_str),
+                );
             } else {
-                show_error(prev_rect.min, format!("First use of ID {}", id_str));
-                show_error(new_rect.min, format!("Second use of ID {}", id_str));
+                show_error(
+                    prev_rect.min,
+                    format!("First use of {} ID {}", what, id_str),
+                );
+                show_error(
+                    new_rect.min,
+                    format!("Second use of {} ID {}", what, id_str),
+                );
             }
         }
     }
@@ -286,7 +305,7 @@ impl Context {
         self.interact_with_hovered(layer_id, id, rect, sense, enabled, hovered)
     }
 
-    /// You specify if a thing is hovered, and the function gives a `Response`.
+    /// You specify if a thing is hovered, and the function gives a [`Response`].
     pub(crate) fn interact_with_hovered(
         &self,
         layer_id: LayerId,
@@ -308,6 +327,7 @@ impl Context {
             hovered,
             clicked: Default::default(),
             double_clicked: Default::default(),
+            triple_clicked: Default::default(),
             dragged: false,
             drag_released: false,
             is_pointer_button_down_on: false,
@@ -321,7 +341,7 @@ impl Context {
             return response;
         }
 
-        self.register_interaction_id(id, rect);
+        self.check_for_id_clash(id, rect, "widget");
 
         let clicked_elsewhere = response.clicked_elsewhere();
         let ctx_impl = &mut *self.write();
@@ -391,6 +411,8 @@ impl Context {
                                 response.clicked[click.button as usize] = clicked;
                                 response.double_clicked[click.button as usize] =
                                     clicked && click.is_double();
+                                response.triple_clicked[click.button as usize] =
+                                    clicked && click.is_triple();
                             }
                         }
                     }
@@ -533,11 +555,28 @@ impl Context {
 
 impl Context {
     /// Call this if there is need to repaint the UI, i.e. if you are showing an animation.
+    ///
     /// If this is called at least once in a frame, then there will be another frame right after this.
     /// Call as many times as you wish, only one repaint will be issued.
+    ///
+    /// If called from outside the UI thread, the UI thread will wake up and run,
+    /// provided the egui integration has set that up via [`Self::set_request_repaint_callback`]
+    /// (this will work on `eframe`).
     pub fn request_repaint(&self) {
         // request two frames of repaint, just to cover some corner cases (frame delays):
-        self.write().repaint_requests = 2;
+        let mut ctx = self.write();
+        ctx.repaint_requests = 2;
+        if let Some(callback) = &ctx.request_repaint_callbacks {
+            (callback)();
+        }
+    }
+
+    /// For integrations: this callback will be called when an egui user calls [`Self::request_repaint`].
+    ///
+    /// This lets you wake up a sleeping UI thread.
+    pub fn set_request_repaint_callback(&self, callback: impl Fn() + Send + Sync + 'static) {
+        let callback = Box::new(callback);
+        self.write().request_repaint_callbacks = Some(callback);
     }
 
     /// Tell `egui` which fonts to use.
@@ -600,7 +639,7 @@ impl Context {
     /// Will become active at the start of the next frame.
     ///
     /// Note that this may be overwritten by input from the integration via [`RawInput::pixels_per_point`].
-    /// For instance, when using `egui_web` the browsers native zoom level will always be used.
+    /// For instance, when using `eframe` on web, the browsers native zoom level will always be used.
     pub fn set_pixels_per_point(&self, pixels_per_point: f32) {
         if pixels_per_point != self.pixels_per_point() {
             self.request_repaint();
@@ -667,8 +706,19 @@ impl Context {
         name: impl Into<String>,
         image: impl Into<ImageData>,
     ) -> TextureHandle {
+        let name = name.into();
+        let image = image.into();
+        let max_texture_side = self.input().max_texture_side;
+        crate::egui_assert!(
+            image.width() <= max_texture_side && image.height() <= max_texture_side,
+            "Texture {:?} has size {}x{}, but the maximum texture side is {}",
+            name,
+            image.width(),
+            image.height(),
+            max_texture_side
+        );
         let tex_mngr = self.tex_manager();
-        let tex_id = tex_mngr.write().alloc(name.into(), image.into());
+        let tex_id = tex_mngr.write().alloc(name, image);
         TextureHandle::new(tex_mngr, tex_id)
     }
 
@@ -773,22 +823,24 @@ impl Context {
     }
 
     /// Tessellate the given shapes into triangle meshes.
-    pub fn tessellate(&self, shapes: Vec<ClippedShape>) -> Vec<ClippedMesh> {
+    pub fn tessellate(&self, shapes: Vec<ClippedShape>) -> Vec<ClippedPrimitive> {
         // A tempting optimization is to reuse the tessellation from last frame if the
         // shapes are the same, but just comparing the shapes takes about 50% of the time
         // it takes to tessellate them, so it is not a worth optimization.
 
-        let mut tessellation_options = *self.tessellation_options();
-        tessellation_options.pixels_per_point = self.pixels_per_point();
-        tessellation_options.aa_size = 1.0 / self.pixels_per_point();
+        let pixels_per_point = self.pixels_per_point();
+        let tessellation_options = *self.tessellation_options();
+        let font_image_size = self.fonts().font_image_size();
+
         let paint_stats = PaintStats::from_shapes(&shapes);
-        let clipped_meshes = tessellator::tessellate_shapes(
-            shapes,
+        let clipped_primitives = tessellator::tessellate_shapes(
+            pixels_per_point,
             tessellation_options,
-            self.fonts().font_image_size(),
+            shapes,
+            font_image_size,
         );
-        self.write().paint_stats = paint_stats.with_clipped_meshes(&clipped_meshes);
-        clipped_meshes
+        self.write().paint_stats = paint_stats.with_clipped_primitives(&clipped_primitives);
+        clipped_primitives
     }
 
     // ---------------------------------------------------------------------
@@ -854,7 +906,7 @@ impl Context {
     /// Latest reported pointer position.
     /// When tapping a touch screen, this will be `None`.
     #[inline(always)]
-    pub(crate) fn latest_pointer_pos(&self) -> Option<Pos2> {
+    pub fn pointer_latest_pos(&self) -> Option<Pos2> {
         self.input().pointer.latest_pos()
     }
 
@@ -895,13 +947,8 @@ impl Context {
         self.memory().layer_id_at(pos, resize_grab_radius_side)
     }
 
-    /// The overall top-most layer. When an area is clicked on or interacted
-    /// with, it is moved above all other areas.
-    pub fn top_most_layer(&self) -> Option<LayerId> {
-        self.memory().top_most_layer()
-    }
-
-    /// Moves the given area to the top.
+    /// Moves the given area to the top in its [`Order`].
+    /// [`Area`]:s and [`Window`]:s also do this automatically when being clicked on or interacted with.
     pub fn move_to_top(&self, layer_id: LayerId) {
         self.memory().areas.move_to_top(layer_id);
     }
@@ -1191,11 +1238,12 @@ impl Context {
         ui.horizontal(|ui| {
             ui.label(format!(
                 "{} collapsing headers",
-                self.data().count::<containers::collapsing_header::State>()
+                self.data()
+                    .count::<containers::collapsing_header::InnerState>()
             ));
             if ui.button("Reset").clicked() {
                 self.data()
-                    .remove_by_type::<containers::collapsing_header::State>();
+                    .remove_by_type::<containers::collapsing_header::InnerState>();
             }
         });
 
@@ -1246,3 +1294,10 @@ impl Context {
         self.set_style(style);
     }
 }
+
+#[cfg(test)]
+#[test]
+fn context_impl_send_sync() {
+    fn assert_send_sync<T: Send + Sync>() {}
+    assert_send_sync::<Context>();
+}
