@@ -1,12 +1,15 @@
+use std::ops::RangeInclusive;
+use std::sync::Arc;
+
 use crate::{
-    emath::{Align2, Pos2, Rect, Vec2},
+    emath::{pos2, Align2, Pos2, Rect, Vec2},
     layers::{LayerId, PaintList, ShapeIdx},
     Color32, Context, FontId,
 };
 use epaint::{
-    mutex::{Arc, RwLockReadGuard, RwLockWriteGuard},
+    mutex::{RwLockReadGuard, RwLockWriteGuard},
     text::{Fonts, Galley},
-    CircleShape, RectShape, Rounding, Shape, Stroke, TextShape,
+    CircleShape, RectShape, Rounding, Shape, Stroke,
 };
 
 /// Helper to paint shapes and text to a specific region on a specific layer.
@@ -20,7 +23,7 @@ pub struct Painter {
     /// Where we paint
     layer_id: LayerId,
 
-    /// Everything painted in this `Painter` will be clipped against this.
+    /// Everything painted in this [`Painter`] will be clipped against this.
     /// This means nothing outside of this rectangle will be visible on screen.
     clip_rect: Rect,
 
@@ -51,6 +54,19 @@ impl Painter {
         }
     }
 
+    /// Create a painter for a sub-region of this [`Painter`].
+    ///
+    /// The clip-rect of the returned [`Painter`] will be the intersection
+    /// of the given rectangle and the `clip_rect()` of the parent [`Painter`].
+    pub fn with_clip_rect(&self, rect: Rect) -> Self {
+        Self {
+            ctx: self.ctx.clone(),
+            layer_id: self.layer_id,
+            clip_rect: rect.intersect(self.clip_rect),
+            fade_to_color: self.fade_to_color,
+        }
+    }
+
     /// Redirect where you are painting.
     pub fn set_layer_id(&mut self, layer_id: LayerId) {
         self.layer_id = layer_id;
@@ -70,10 +86,7 @@ impl Painter {
         self.fade_to_color = Some(Color32::TRANSPARENT);
     }
 
-    /// Create a painter for a sub-region of this `Painter`.
-    ///
-    /// The clip-rect of the returned `Painter` will be the intersection
-    /// of the given rectangle and the `clip_rect()` of this `Painter`.
+    #[deprecated = "Use Painter::with_clip_rect"] // Deprecated in 2022-04-18, before egui 0.18
     pub fn sub_region(&self, rect: Rect) -> Self {
         Self {
             ctx: self.ctx.clone(),
@@ -104,14 +117,14 @@ impl Painter {
         self.layer_id
     }
 
-    /// Everything painted in this `Painter` will be clipped against this.
+    /// Everything painted in this [`Painter`] will be clipped against this.
     /// This means nothing outside of this rectangle will be visible on screen.
     #[inline(always)]
     pub fn clip_rect(&self) -> Rect {
         self.clip_rect
     }
 
-    /// Everything painted in this `Painter` will be clipped against this.
+    /// Everything painted in this [`Painter`] will be clipped against this.
     /// This means nothing outside of this rectangle will be visible on screen.
     #[inline(always)]
     pub fn set_clip_rect(&mut self, clip_rect: Rect) {
@@ -194,7 +207,7 @@ impl Painter {
 /// ## Debug painting
 impl Painter {
     #[allow(clippy::needless_pass_by_value)]
-    pub fn debug_rect(&mut self, rect: Rect, color: Color32, text: impl ToString) {
+    pub fn debug_rect(&self, rect: Rect, color: Color32, text: impl ToString) {
         self.rect_stroke(rect, 0.0, (1.0, color));
         self.text(
             rect.min,
@@ -233,8 +246,7 @@ impl Painter {
 
 /// # Paint different primitives
 impl Painter {
-    /// Paints the line from the first point to the second using the `stroke`
-    /// for outlining shape.
+    /// Paints a line from the first point to the second.
     pub fn line_segment(&self, points: [Pos2; 2], stroke: impl Into<Stroke>) {
         self.add(Shape::LineSegment {
             points,
@@ -242,6 +254,22 @@ impl Painter {
         });
     }
 
+    /// Paints a horizontal line.
+    pub fn hline(&self, x: RangeInclusive<f32>, y: f32, stroke: impl Into<Stroke>) {
+        self.add(Shape::LineSegment {
+            points: [pos2(*x.start(), y), pos2(*x.end(), y)],
+            stroke: stroke.into(),
+        });
+    }
+
+    /// Paints a vertical line.
+    pub fn vline(&self, x: f32, y: RangeInclusive<f32>, stroke: impl Into<Stroke>) {
+        self.add(Shape::LineSegment {
+            points: [pos2(x, *y.start()), pos2(x, *y.end())],
+            stroke: stroke.into(),
+        });
+    }
+
     pub fn circle(
         &self,
         center: Pos2,
@@ -385,7 +413,7 @@ impl Painter {
 
     /// Paint text that has already been layed out in a [`Galley`].
     ///
-    /// You can create the `Galley` with [`Self::layout`].
+    /// You can create the [`Galley`] with [`Self::layout`].
     ///
     /// If you want to change the color of the text, use [`Self::galley_with_color`].
     #[inline(always)]
@@ -397,16 +425,13 @@ impl Painter {
 
     /// Paint text that has already been layed out in a [`Galley`].
     ///
-    /// You can create the `Galley` with [`Self::layout`].
+    /// You can create the [`Galley`] with [`Self::layout`].
     ///
     /// The text color in the [`Galley`] will be replaced with the given color.
     #[inline(always)]
     pub fn galley_with_color(&self, pos: Pos2, galley: Arc<Galley>, text_color: Color32) {
         if !galley.is_empty() {
-            self.add(TextShape {
-                override_text_color: Some(text_color),
-                ..TextShape::new(pos, galley)
-            });
+            self.add(Shape::galley_with_color(pos, galley, text_color));
         }
     }
 }
